<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * This file is part of the PEAR Testing_DocTest package.
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to the MIT license that is available
 * through the world-wide-web at the following URI:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category  Testing 
 * @package   Testing_DocTest
 * @author    David JEAN LOUIS <izimobil@gmail.com>
 * @copyright 2008 David JEAN LOUIS
 * @license   http://opensource.org/licenses/mit-license.php MIT License 
 * @version   CVS: $Id$
 * @link      http://pear.php.net/package/Testing_DocTest
 * @since     File available since release 0.1.0
 * @filesource
 */

/**
 * Required file
 */
require_once 'Testing/DocTest/OutputterInterface.php';

/**
 * DocTest Outputter default class.
 *
 * @category  Testing
 * @package   Testing_DocTest
 * @author    David JEAN LOUIS <izimobil@gmail.com>
 * @copyright 2008 David JEAN LOUIS
 * @license   http://opensource.org/licenses/mit-license.php MIT License 
 * @version   Release: 0.3.0
 * @link      http://pear.php.net/package/Testing_DocTest
 * @since     Class available since release 0.1.0
 */
class Testing_DocTest_Outputter_Default implements Testing_DocTest_OutputterInterface
{
    // properties {{{

    /**
     * Timer beginning time.
     *
     * @var float time
     * @access private
     */
    private $_time = 0;

    /**
     * Opened log file.
     *
     * @var resource $_logfile
     * @access private
     */
    private $_logfile = false;

    // }}}
    // __construct() {{{

    /**
     * Constructor.
     *
     * @access public
     */
    public function __construct()
    {
        $reg = Testing_DocTest_Registry::singleton();
        if ($reg->logfile) {
            $this->_logfile = fopen($reg->logfile, 'wb');
        }
        $time        = explode(' ', microtime());
        $this->_time = $time[1] + $time[0];
    }

    // }}}
    // __destruct() {{{

    /**
     * Destructor.
     * Will eventually close the opened logfile.
     *
     * @access public
     */
    public function __destruct()
    {
        if (false !== $this->_logfile && is_resource($this->_logfile)) {
            fclose($this->_logfile);
        }
    }

    // }}}
    // begin() {{{

    /**
     * Begin of the DocTest session.
     *
     * @param array $files a multidimensional array of file=>testcase array 
     *
     * @access public
     * @return void
     */
    public function begin(array $files)
    {
        if (empty($files)) {
            $this->_output("Nothing to process.\n");
        }
    }

    // }}}
    // end() {{{

    /**
     * End of the DocTest session.
     *
     * @param array $files a multidimensional array of file=>testcase array 
     *
     * @access public
     * @return void
     */
    public function end(array $files)
    {
        if (empty($files)) {
            return;
        }
        $time   = explode(' ', microtime());
        $t      = ($time[1] + $time[0]) - $this->_time;
        $passed = $skipped = $failed = $error = 0;
        foreach ($files as $file=>$tbArray) {
            foreach ($tbArray as $tb) {
                if ($tb->state == Testing_DocTest_TestCase::STATE_SKIPPED) {
                    $skipped++;
                } else if (
                    $tb->state == Testing_DocTest_TestCase::STATE_PASSED) {
                    $passed++;
                } else {
                    $failed++;
                }
            }
        }
        $this->_output(sprintf("\nTotal time    : %.4f sec.\n", $t));
        $this->_output("Passed tests  : $passed\n");
        $this->_output("Skipped tests : $skipped\n");
        $this->_output("Failed tests  : $failed\n\n");
    }

    // }}}
    // beginFile() {{{

    /**
     * Begin of file parsing.
     *
     * @param string $file          path to the file that will be processed
     * @param array  $testCaseArray an array of test case instances
     *
     * @access public
     * @return void
     */
    public function beginFile($file, array $testCaseArray)
    {
    }

    // }}}
    // endFile() {{{

    /**
     * End of file parsing.
     *
     * @param string $file          path to the file that will be processed
     * @param array  $testCaseArray an array of test case instances
     *
     * @access public
     * @return void
     */
    public function endFile($file, array $testCaseArray)
    {
    }

    // }}}
    // beginTestCase() {{{

    /**
     * Begin of test case parsing.
     *
     * @param object $testCase a test case instance
     *
     * @access public
     * @return void
     */
    public function beginTestCase(Testing_DocTest_TestCase $testCase)
    {
    }

    // }}}
    // endTestCase() {{{

    /**
     * End of test case parsing.
     *
     * @param object $testCase a test case instance
     *
     * @access public
     * @return void
     */
    public function endTestCase(Testing_DocTest_TestCase $testCase)
    {
        $name = sprintf("%s in file \"%s\"\n", 
            $testCase->name, $testCase->file);
        if ($testCase->state == Testing_DocTest_TestCase::STATE_SKIPPED) {
            $this->_output('[SKIP]  ', 33);
            $this->_output($name);
        } else {
            if ($testCase->state == Testing_DocTest_TestCase::STATE_PASSED) {
                $this->_output('[PASS]  ', 32);
                $this->_output($name);
            } else {
                if ($testCase->state == Testing_DocTest_TestCase::STATE_ERROR) {
                    $this->_output('[ERROR] ', 31, true);
                } else {
                    $this->_output('[FAIL]  ', 31, true);
                }
                $this->_output($name, false, true);
                $this->_output(str_repeat('-', 31) . " Expected " 
                    . str_repeat('-', 31) . "\n", false, true);
                $this->_output($testCase->expectedValue . "\n", false, true);
                $this->_output(str_repeat('-', 31) . "  Actual  " 
                    . str_repeat('-', 31) . "\n", false, true);
                $this->_output($testCase->actualValue . "\n", false, true);
                $this->_output(str_repeat('-', 72) . "\n", false, true);
            }
        }
    }

    // }}}
    // _output() {{{

    /**
     * Writes the message $msg to STDOUT or to the logfile.
     *
     * @param string $msg   the message to output
     * @param int    $color the color code (optional)
     * @param bool   $force force output even if quiet mode
     *
     * @return void
     * @access private
     */
    private function _output($msg, $color=false, $force=false)
    {
        $reg = Testing_DocTest_Registry::singleton();
        if ($reg->quiet && !$force) {
            return;
        }
        if ($color && !$reg->no_colors && !$this->_logfile) {
            echo "\033[{$color}m" . $msg . "\033[0;0m";
        } else {
            if ($this->_logfile) {
                fwrite($this->_logfile, $msg);
            } else {
                echo $msg;
            }
        }
    }

    // }}}
}
